# Contribution

Thank you for contributing to `purescript-argonaut-traversals`!

Following these guidelines helps ensure we're able to carefully consider your contribution and help you finalize your pull request. Your time (like ours) is valuable, and we try to at least provide constructive feedback on every contribution.

### Contributions we love
`purescript-argonaut-traversals` is an open-source library and we love to receive contributions. There are many ways you can contribute:

* Help expand our test coverage
* Help fix open issues, even if that simply means adding a helpful comment
* Help push along open pull requests that need more work to be complete
* Improve or add new usage examples to better demonstrate how to use the library in real-world scenarios
* Update documentation and tutorials to be easier to understand, more comprehensive, and above all -- up to date!

### Contributions we should discuss
Some contributions will take some discussion before we accept an update to the project. If your contribution includes one of the below examples (or seems to be reasonably similar), please consider reaching out to us before putting in a lot of work to build the feature. We're active on the [PureScript user forum](https://discourse.purescript.org) and the [functional programming Slack](https://functionalprogramming.slack.com/) community (new? [use this link to join](https://fpchat-invite.herokuapp.com/)!). For feature requests, feel free to open an issue with a tag.

* New features and functions that will change the library's public API
* New tutorials (while we love seeing new tutorials, we won't always be able to feature them in the readme)

# Ground Rules

We have a small set of quality-of-life guidelines for contributing to `purescript-argonaut-traversals`. These include:

* All pull requests must pass continuous integration.
* If you are adding new functionality, you should provide tests and documentation for your code. If you're fixing an existing bug, please provide a failing test case your patch solves.
* If possible, please avoid requiring new dependencies.

### Filing issues
If you have a general question about the project, it's best to ask on the [PureScript user forum](https://discourse.purescript.org) or the [functional programming Slack](https://functionalprogramming.slack.com/) than to open a new issue. If you have run into a bug in the project, then please do open an issue! When you do, we ask that you follow a few steps which are outlined in our issues template. The gist of it is here:

* Verify the problem is indeed with `purescript-argonaut-traversals` (not with Pulp, Bower, or PureScript);
* Record what versions you are using for PureScript and `purescript-argonaut-traversals`
* Describe the issue with steps to reproduce (as much as you are able). A minimal reproducible example is the absolute best case scenario.

We promise to address the issue as soon as we can.

### Suggesting features or enhancements
We love to hear about ways we could make  better. If you're wishing for a feature that doesn't exist in `purescript-argonaut-traversals`, you're probably not alone; there are bound to be others with similar needs. Please feel free to open an issue on GitHub that describes:

* the feature you would like to see
* why you need it
* how it should work

We promise to review your issue, but we aren't always able to accommodate all requests. It helps if you're able to contribute to the implementation, too!

### Code reviews
The core team looks at pull requests weekly, at which point we will review your code, ensure it meets our ground rules and fits with the philosophy of the project, and -- if necessary -- provide constructive feedback. As soon as at least one member of the team has signed off on your pull request, we will merge your contribution.

If you opened a pull request and we need to ask a few questions, we expect responses within a month. After that we may close the pull request if it isn't showing any activity to keep things manageable.
